-- AutoCAD Array Command Emulation 
	
fn getRectangularOrPolar prompt: default:#rectangular  = 
(
	local CarryOn = true
	local TR
	local a,b
	local Val
	local EEnable
	
	EEnable = EscapeEnable
	EscapeEnable = false
	
	do 
	(
		setListenerSel #(-1,-1)
		TR = getKBline prompt:prompt
		a = getListenerSel()
		case of 
		(
			(TR==#Escape): 												(SetListenerSel #(-1,-1) ; return #Escape)	-- Escape key press
			(TR=="r" or TR=="R"): 											(CarryOn = false ; Val = #rectangular)	-- R key press
			(TR=="rectangular" or TR=="RECTANGULAR" or TR=="Rectangular"):	(CarryOn = false ; Val = #rectangular)	-- Rectangular typed
			(TR=="P" or TR=="p"):   										(CarryOn = false ; Val = #polar)		-- P key press
			(TR=="polar" or TR=="POLAR" or TR=="Polar"):   					(CarryOn = false ; Val = #polar)		-- Polar typed
			(TR==""):														(CarryOn = false ; Val = default)		-- Enter key
			default:
				( 
					b = getListenerSel()
					setListenerSel #((a[1]),b[1])
					SetListenerSelText "Invalid entry... \n" 
					CarryOn = true
				)
		)
	) while CarryOn
	
	EscapeEnable = EEnable
	Val
)

-- ARRAY FUNCTION *******************************************************************************************
fn _array = 
(
	-- DECLARE LOCALS
	local Item
	local ArrayType
	local ArrayPlaneTM,IArrayPlaneTM
	local TR
	local UserGrid
	local ViewTM,ViewType
	local RowSpace,ColSpace
	local FirstItem = true
	local a,b
	local NumRows
	local TForm
	local ArrayGroup
	local NewPt = [0,0,0]
	local BasePt
	local PolarArray
	local m1,m2,m3,m4,m5,m6
	local BaseTM,nBaseTM
	
	-- SET SYSTEM PARAMETERS
	clearlistener()
	EscapeEnable = false
	
	-- SELECT OBJECTS TO ARRAY *****************************************************************************
	-- Here we get objects to perform array on. Note that we also get and save the viewport
	-- used to select the first object. This is used later to determine the array plane.
	
	max select none
	while true do
	(
		Item = pickObject prompt:"Select Objects:" count:1
		
		if Item == #Escape do
		(
			max select none
			EscapeEnable = true
			redrawviews()
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
		
		if Item == undefined do	-- Corresponding to enter or right mouse click
		(
			format "\n"
			exit
		)
		
		selectmore Item
		if FirstItem do 
		(
			ViewTM = getViewTM()
			ViewType = view_type()
			FirstItem = false
		)

		redrawviews()
		format "\n"
	)

	-- TEST IF SELECTION SET IS EMPTY, IF SO RETURN
	if selection.count == 0 do
	(
		max select none
		EscapeEnable = true
		setListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)
	
	-- COLLECT SELECTED ITEMS INTO ARRAY
	ArrayGroup = selection as array

	-- PROMPT FOR ARRAY TYPE; RECTANGULAR OR POLAR 
	if (ArrayType = getRectangularOrPolar prompt:"Rectangular or Polar array (<R>/P):" default:#rectangular) == #Escape do
	(
		EscapeEnable = true
		max select none
		setListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)

	-- DETERMINE PLANE OF ARRAY ****************************************************************************
	-- In this section of the code we determine the "ARRAY PLANE", i.e., the plane the array will be
 	-- performed in. The following procedure is used.
	-- If there is a user grid defined AND active...the array plane will be the grid plane.
	-- If there is not a user grid defined the array plane is based on the viewport in which
	-- the original object was selected. The array plane is the principal plane parallel to the viewing plane
	-- selected, EXCEPT if a user or perspective viewport is selected. Then the array plane is taken as
	-- the World XY principal plane.
	
	-- DETERMINE IF A USER DEFINED GRID EXISTS AND IS ACTIVE
	a = getCPTM()	
	b = matrix3 1
	if (a.row1==b.row1 and a.row2==b.row2 and a.row3==b.row3 and a.row4==b.row4) 
	then UserGrid = false
	else UserGrid = true
	
	-- SET UP ARRAY PLANE TRANSFORMATION MATRIX
	if UserGrid 
	then 
	( 
		ArrayPlaneTM = inverse a
		TForm = true
	)
	else 
	(
		case ViewType of 
		(
			#perspective: 	TForm = false 
			#user:			TForm = false 
			#top:			Tform = false 
			default:		TForm = true 
		)
		ArrayPlaneTM = ViewTM
	)
	
	-- CODE SECTION FOR RECTANGULAR ARRAY *******************************************************************************
	if ArrayType == #Rectangular then
	(
		-- PROMPT FOR NUMBER OF ROWS
		if (numRows = getInt prompt: "Number of rows (---) <1>:" minval:1 maxval:300 default:1 ) == #Escape do
		(
			EscapeEnable = true
			max select none
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
		
		-- PROMPT FOR NUMBER OF COLUMNS
		if (numCols = getInt prompt: "Number of columns (|||) <1>:" minval:1 maxval:300 default:1 ) == #Escape do
		(
			EscapeEnable = true
			max select none
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)	
		
		-- ENSURE NUMBER OF COLUMNS AND ROWS ARE NOT BOTH ONE
		if (numRows == 1 and numCols == 1) do
		(
			EscapeEnable = true
			max select none
			SetListenerSel #(-1,-1)
			return (format "One-element array, nothing to do.\nOK\n")
		)
		
		-- PROMPT FOR SPACING
		if (RowSpace = getReal prompt: "Distance between rows (---):" minval:0 maxval:1E10 default:10) == #Escape do
		(
			EscapeEnable = true
			max select none
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
			
		if (ColSpace = getReal prompt: "Distance between columns (|||):" minval:0 maxval:1E10 default:10) == #Escape do
		(
			EscapeEnable = true
			max select none
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
		
		-- FOR EACH ITEM IN THE SELECTION SET PERFORM ARRAY 
		setWaitCursor()
		for Item in ArrayGroup do
		(
			if TForm 
			then BasePt = Item.pos * ArrayPlaneTM
			else BasePt = Item.pos
			
			for i = 0 to (numRows-1) do
			(
				for j = 0 to (numCols-1) do
				(
					newPt.x = BasePt.x + j * ColSpace
					newPt.y = BasePt.y + i * RowSpace
					newPt.z = BasePt.z
					
					if TForm do ( newPt = newPt * (inverse ArrayPlaneTM) )
					
					if ( i == 0 and j == 0 ) then ()
					else
					(
						newItem = copy Item
						newItem.pos = newPt
						newItem.wirecolor =  Item.wirecolor
					)
				)
			)
		)
		setArrowCursor()
	)
	-- CODE SECTION FOR POLAR ARRAY *******************************************************************************
	else -- ARRAY TYPE IS POLAR
	(
		-- PROMPT FOR ARRAY CENTER POINT
		CenterPt = pickpoint prompt:"Specify center point of array:"
		if (ClassOf CenterPt != Point3) do
		(
			EscapeEnable = true
			max select none 
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n" )
		)			
		format "\n"

		-- IF TFORM IS TRUE .... TRANSFORM ARRAY CENTER POINT TO ARRAY PLANE
		if TForm do CenterPt = CenterPt * ArrayPlaneTM

		-- PROMPT FOR NUMBER OF ITEMS IN THE POLAR ARRAY
		if (numPolar = getInt prompt:"Number of items: " minval:2 maxval:1000 default:2) == #Escape do
		(
			EscapeEnable = true
			max select none
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
		
		-- PROMPT FOR THE ANGLE TO FILL 
		if (FillAngle = getReal prompt: "Angle to fill (+=ccw, -=cw) <360>:" minval: -360 maxval:360 default:360) == #Escape do
		(
			EscapeEnable = true
			max select none
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
		
		
		-- PROMPT TO DETERMINE WHETHER OBJECTS SHOULD BE ROTATED AS THEY ARE ARRAYED
		if (PolarRotate = getYN prompt:"Rotate objects as they are copied? <Y>" default:true)==#Escape do
		(
			EscapeEnable = true
			max select none
			setListenerSel #(-1,-1)
			return format "*Cancel*\nOK\n"
		)
		
		-- DETERMINE DELTA ANGLE BETWEEN ITEMS
		if FillAngle == 360 then	DeltaAng = 360.0/(numPolar + 0.0)
		else	DeltaAng = FillAngle/(numPolar - 1.0)
	
		-- TAKE INVERSE OF ARRAY PLANE FOR LATER USE IN MAPPING FROM ARRAY PLANE TO WORLD COORDS
		IArrayPlaneTM = (inverse ArrayPlaneTM)	
		
		-- FOR EACH ITEM IN THE SELECTION SET PERFORM ARRAY 
		setWaitCursor()
		for Item in ArrayGroup do
		(
			-- GET ITEM POSITION AND TRANSFORM TO ARRAY PLANE 
			if TForm 
			then BaseTM = Item.transform * ArrayPlaneTM
			else BaseTM = Item.transform
		
			for i = 1 to (numPolar-1) do
			(
				if PolarRotate then
				(
					nBaseTM = BaseTM * transmatrix (-1 * CenterPt) * rotateZMatrix (DeltaAng * i) * transmatrix CenterPt
				)
				else
				(	
					m1 = transmatrix ( -1 * BaseTM.row4)
					m2 = rotateZMatrix (-1 * DeltaAng * i)
					m3 = transmatrix BaseTM.row4
					m4 = transmatrix (-1 * CenterPt) 
					m5 = rotateZMatrix (DeltaAng * i)
					m6 = transmatrix CenterPt
										
					nBaseTM = BaseTM * m1 * m2 * m3 * m4 * m5 * m6
				)
				
				if TForm do nBaseTM = nBaseTM * IArrayPlaneTM
				
				NewItem = copy Item
				NewItem.transform = nBaseTM
				NewItem.wirecolor = Item.wirecolor
			)
		)
		setArrowCursor()
	)
	
	-- RESET SYSTEM PARAMETERS
	EscapeEnable = true
	max select none
	setListenerSel #(-1,-1)
	format "OK\n"
)