-- AUTOCAD CIRCLE COMMAND SHORTCUT 

-- MOUSE CALLBACK FUNCTIONS *******************************************************************
-- HANDLER: THREE POINT CIRCLE
fn CircleCB2 P3 A =
(
	local M2 = #()
	local CircleRad
	local CenterPt
	local WorldCenterPt
	
	-- EXTRACT VARIABLES FROM ARRAY
	CircleID = A[1]
	P1 = A[2]
	P2 = A[3]
	M1 = A[4]
	TForm = A[5]
	WorkTM = A[6]
	IWorkTM = A[7]
	
	if TForm do (P3 = P3 * WorkTM)
	
	M2[1] = (P1 + P3)/2.0
	
	M2[2] = normalize (cross  z_axis  [(M2[1].x-P1.x),(M2[1].y-P1.y),0.0] )
	
		
	-- TEST IF P1,P2 AND P3 ARE COLINEAR, IF SO RETURN
	if  ( abs(dot M1[2] M2[2]) > 0.9998 ) 
	then () -- if colinear don't try to create
	else
	(
		-- DETERMINE CIRCLE CENTER POINT (INTERSECTION OF TWO PERPENDICULAR BISECTORS)
		CenterPt = LineIntersect M1[1] (M1[1]+M1[2]) M2[1] (M2[1]+M2[2]) 
		
		-- DETERMINE CIRCLE RADIUS
		CircleRad = length (CenterPt - P1)
		
		if TForm then
		(
			WorldCenterPt = CenterPt * IWorkTM
			CircleID.transform = matrix3 IWorkTM.row1 IWorkTM.row2 IWorkTM.row3 WorldCenterPt
		)
		else 
		(
			circleID.pos = CenterPt
		)
	
		CircleID.radius = CircleRad
		redrawviews()
	)
)

-- HANDLER: TWO POINT CIRCLE
fn CircleCB3 P2 A =
(
	local CircleRad
	local CenterPt
	local WorldCenterPt
		
	CircleID = A[1]
	P1 = A[2]
	TForm = A[3]
	WorkTM = A[4]
	IWorkTM = A[5]
	
	-- TRANSFORM P2 IF REQUIRED		
	if TForm do (P2 = P2 * WorkTM)

	CenterPt = (P1 + P2)/2.0
	CircleRad = length(CenterPt - P1)
	
	if TForm then
	(
		WorldCenterPt = CenterPt * IWorkTM
		CircleID.transform = matrix3 IWorkTM.row1 IWorkTM.row2 IWorkTM.row3 WorldCenterPt
	)
	else 
	(
		circleID.pos = CenterPt
	)

	CircleID.radius = CircleRad
	
	redrawviews()
)

-- HANDLER: CENTER POINT AND POINT ON RADIUS
fn CircleCB5 pt A = 
(
	A[1].radius = length(A[2] - pt)
)

-- WORKING COORDINATE SYSTEM TRANSFORMATION MATRIX ********************************************
fn getWorkTM =
(
	local a,b
	local UserGrid
	local TForm
	local WorkTM,IWorkTM
	
	-- DETERMINE IF USER GRID EXISTS AND IS ACTIVE
	a = getCPTM()	
	b = matrix3 1
	if (a.row1==b.row1 and a.row2==b.row2 and a.row3==b.row3 and a.row4==b.row4) 
	then UserGrid = false
	else UserGrid = true
	
	-- DETERMINE WHAT TRANSFORMATION IS REQD (IF NOT REQD SET TO WCS; MATRIX 3 1)
	if UserGrid 
	then 
	( 
		WorkTM = inverse a
		TForm = true
	)
	else 
	(
		case view_type() of 
		(
			#perspective: 	( TForm = false ; WorkTM = matrix3 1 )
			#user:			( TForm = false ; WorkTM = matrix3 1 )
			#top:			( Tform = false ; WorkTM = matrix3 1 )
			default:		( TForm = true ; WorkTM = getViewTM())
		)
	)
	
	if TForm 
	then IWorkTM = inverse WorkTM
	else IWorkTM = matrix3 1
		
	return #(Tform,WorkTM,IWorkTM)
)

-- CIRCLE FUNCTION ************************************************************************
fn _circle =
(
	-- DECLARE LOCALS
	local WorkTM,IWorkTM,TForm = false
	local CircleID
	local CenterPt,WorldCenterPt,LocalCenterPt
	local CircleRadiusPt
	local CircleRad
	local TolVal = 0.0005
	local CarryOn = true
	local R,RtnCode
	local P1,P2,P3
	local M1=#(),M2=#()
	local ptMarkers = #()
	
	-- SYSTEM SETTINGS
	EscapeEnable = false
	clearListener()
	
	-- GET CIRCLE CENTER POINT
	R = pickpoint prompt:"CIRCLE 3P/2P/<Center point>:" terminators:#()
	
	if (Classof R != Array) do
	(
		EscapeEnable=true
		setListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)
		
	case of
	(
		(classof R[1]==point3 and R[2]==undefined):								RtnCode = 5 -- Regular point input
		(R[1]==#RightClick and R[2]==undefined):								RtnCode = 1 -- Right Mouse Click
		(R[1]==#Escape and R[2]==undefined):									RtnCode = 1	-- Escape key pressed
		((R[1]=="3P" or R[1]=="3p" or R[1]=="3") and R[2]==undefined):			RtnCode = 2 -- 3P option
		((R[1]=="2P" or R[1]=="2p" or R[1]=="2") and R[2]==undefined):			RtnCode = 3 -- 2P option
		default:																RtnCode = 1
	)
	
	-- ESCAPE PRESSED *************************************************************
	if RtnCode == 1 do
	(
		EscapeEnable = true
		setListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)

	-- THREE POINT CIRCLE DEFINITION **********************************************
	if RtnCode == 2 do
	(
		-- PROMPT FOR FIRST POINT
		P1 = pickpoint prompt:"First point:"
		
		if (Classof P1 != Point3) do
		(
			EscapeEnable = true 
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n" )
		)
		format "\n"
		
		-- GET INFO RELATING TO WORKING COORDINATE SYSTEM
		WorkData = getWorkTM()
		TForm = WorkData[1]
		WorkTM = WorkData[2]
		IWorkTM = WorkData[3]

		-- ADD FIRST POINT MARKER
		append ptMarkers (point pos:P1)
		redrawviews()
		
		-- TRANSFORM P1 IF REQUIRED
		if TForm do (P1 = PointConditioner testpt:(P1 * WorkTM) tolerance:TolVal)
		
		-- PROMPT FOR SECOND POINT
		P2 = pickpoint prompt:"Second point:"
			
		if (Classof P2 != Point3) do
		(
			delete ptMarkers
			redrawviews()
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
		format "\n"
		
		-- ADD SECOND POINT MARKER
		append ptMarkers (point pos:P2)
		redrawviews()
		
		-- TRANSFORM P2 IF REQUIRED
		if TForm do (P2 = PointConditioner testpt:(P2 * WorkTM) tolerance:TolVal)
		
		-- DETERMINE FIRST PERPENDICULAR BISECTOR
		M1[1] = (P1+P2)/2.0
 		M1[2] = normalize(cross  z_axis  [(M1[1].x-P1.x),(M1[1].y-P1.y),0.0])
		
		-- CREATE CIRCLE
		CircleID = Circle pos:[0,0,0] radius:0
		
		-- PROMPT FOR THIRD POINT
		P3=pickpoint prompt:"Third point:" MouseMoveCallBack:#(CircleCB2,#(CircleID,P1,P2,M1,TForm,WorkTM,IWorkTM))
	
		if (Classof P3 != Point3) do
		(
			delete ptMarkers
			delete CircleID
			redrawviews()
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
		format "\n"	
					
		if TForm do (P3 = PointConditioner testpt:(P3 * WorkTM) tolerance:TolVal)
		
		-- DETERMINE SECOND PERPENDICULAR BISECTOR
		-- We create it from either P1P3 OR P2P3,
		-- whichever is longer, (this yields better accuracy)
		D1 = length (P3 - P1)
		D2 = length (P3 - P2)
		
		if D1 >= D2 then 
		(
			M2[1] = (P1 + P3)/2.0
			M2[2] = normalize (cross  z_axis  [(M2[1].x-P1.x),(M2[1].y-P1.y),0.0] )
		)
		else
		(
			M2[1] = (P2 + P3)/2.0
			M2[2] = normalize (cross  z_axis  [(M2[1].x-P2.x),(M2[1].y-P2.y),0.0] )
		)
		
		-- TEST IF P1,P2 AND P3 ARE COLINEAR, IF SO RETURN
		if  ( abs(dot M1[2] M2[2]) > 0.9998 ) do
		(
			delete ptMarkers
			delete CircleID
			redrawviews()
			setListenerSel #(-1,-1)
			return (format "Points are colinear\nOK\n")
		)
		
		-- DETERMINE CIRCLE CENTER POINT (INTERSECTION OF TWO PERPENDICULAR BISECTORS)
		CenterPt = LineIntersect M1[1] (M1[1]+M1[2]) M2[1] (M2[1]+M2[2]) 
		
		-- DETERMINE CIRCLE RADIUS
		CircleRad = length (CenterPt - P1)
		
		if TForm then
		(
			WorldCenterPt = PointConditioner testpt:(CenterPt * IWorkTM) tolerance:TolVal
			CircleID.transform = matrix3 IWorkTM.row1 IWorkTM.row2 IWorkTM.row3 WorldCenterPt
		)
		else 
		(
			circleID.pos = CenterPt
		)
	
		CircleID.radius = CircleRad
		
		delete ptMarkers
		redrawviews()
		SetListenerSel #(-1,-1)
		return format "OK\n"
	)

	-- TWO POINT CIRCLE DEFINITION ************************************************
	if RtnCode == 3 do
	(
		-- PROMPT FOR FIRST POINT
		P1 = pickpoint prompt:"First point:"
		if (Classof P1!= Point3) do
		(
			EscapeEnable = true 
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n" )
		)
		format "\n"
		
		-- GET INFO RELATING TO WORKING COORDINATE SYSTEM
		WorkData = getWorkTM()
		TForm = WorkData[1]
		WorkTM = WorkData[2]
		IWorkTM = WorkData[3]
	
		-- CREATE POINT MARKER FOR FIRST POINT
		append ptMarkers (point pos:P1)
	
		if TForm do (P1 = PointConditioner testpt:(P1 * WorkTM) tolerance:TolVal)
		
		-- CREATE CIRCLE
		CircleID = Circle pos:[0,0,0] radius:0
			
		-- PROMPT FOR SECOND POINT
		P2=pickpoint prompt:"Second point on diameter:" MouseMoveCallBack:#(CircleCB3,#(CircleID,P1,TForm,WorkTM,IWorkTM))
		if (classof P2 != Point3) do
		(
			delete ptMarkers
			delete CircleID
			redrawviews()
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)
		format "\n"	
				
		if TForm do (P2 = PointConditioner testpt:(P2 * WorkTM) tolerance:TolVal)

		CenterPt = (P1 + P2)/2.0
		CircleRad = length(CenterPt - P1)
		
		if TForm then
		(
			WorldCenterPt = PointConditioner testpt:(CenterPt * IWorkTM) tolerance:TolVal
			CircleID.transform = matrix3 IWorkTM.row1 IWorkTM.row2 IWorkTM.row3 WorldCenterPt
		)
		else 
		(
			circleID.pos = CenterPt
		)
	
		CircleID.radius = CircleRad
		
		delete ptMarkers
		redrawviews()
		SetListenerSel #(-1,-1)
		return format "OK\n"
	)

	-- DEFAULT CIRCLE DEFINITION ********************************************************************
	if RtnCode == 5 do
	(
		CenterPt = R[1]
		
		-- GET INFO RELATING TO WORKING COORDINATE SYSTEM
		WorkData = getWorkTM()
		if WorkData[1] then 
		TForm = WorkData[1]
		WorkTM = WorkData[2]
		IWorkTM = WorkData[3]

		-- CREATE "EMPTY" CIRCLE
		CircleID = Circle pos:[0,0,0] radius:0
	
		-- DETERMINE CIRCLE TRANSFORM
		if TForm then  
		(
			IWorkTM = inverse WorkTM
			LocalCenterPt = PointConditioner testpt:(CenterPt * IWorkTM) tolerance:TolVal
			CircleID.transform = matrix3 IWorkTM.row1 IWorkTM.row2 IWorkTM.row3 CenterPt
		)
		else
		(
			LocalCenterPt = CenterPt
			CircleID.pos = LocalCenterPt
		)
		
		-- PROMPT FOR POINT ON CIRCLE RADIUS
		format "\n"
		SetListenerSel #(-1,-1)
		CircleRadiusPt = pickpoint prompt:"Radius:" rubberband:CenterPt mouseMoveCallback:#(CircleCB5,#(CircleID,CenterPt))
	
		if (Classof CircleRadiusPt == string) do
		(
			try(CircleRadiusPt = CenterPt + (CircleRadiusPt as float) * normalize CenterPt)
			catch(return format "*Cancel*\nOK")
		)
			
		if (ClassOf CircleRadiusPt != Point3) do
		(	
			EscapeEnable = true
			delete CircleID
			setListenerSel #(-1,-1)
			return (format "*Cancel*\nOK\n")
		)		
		format "\n"
		
		-- COMPLETE CIRCLE DEFINITION BY ASSIGNING RADIUS
		if TForm do	(CircleRadiusPt = PointConditioner testpt:(CircleRadiusPt * IWorkTM) tolerance:TolVal)
		CircleID.radius = length(CircleRadiusPt - LocalCenterPt)
		
		
		-- CLEAN UP VIEWPORTS AND SEND "OK" TO LISTENER
		Max views redraw
		setListenerSel #(-1,-1)
		format "OK\n"
		EscapeEnable = true
	)
)
		

	
	