--Utility Function Library

-- GET INTEGER FUNCTION *************************************************************************************
-- GetInt function prompts for integer value using the supplied prompt
-- Returns 	typed integer value.
--			undefined if user presses escape key.
--			supplied default if user presses enter key.
-- 			0 if user presses enter key and no default value was supplied.
fn getInt prompt:"Enter integer:"  minval: -32766 maxval:32767  default:0  =
(
	local CarryOn = true
	local a,b
	local EEnable
	
	EEnable = EscapeEnable
	EscapeEnable = false
	
	while CarryOn do
	(
		a = getKBLine  prompt: prompt
		
		-- DETERMINE IF KEYBOARD ENTRY WAS EITHER ESCAPE OR ENTER
		case of 
		(
			(a == #Escape): return #Escape		-- Escape key pressed
			(a == ""): return default			-- Enter key pressed...
			default: CarryOn = false			-- OK so far...
		)
		
		-- TRY TO CONVERT ENTERED VALUE TO AN INTEGER
		try(b = a as integer) catch (CarryOn = true) 
		
		if (b != undefined and not CarryOn) then
		(
			if (b < minval or b > maxval)
			then
			(	
				SetListenerSel #(-1,-1)
				format "Requires an integer between % and %. \n" minval maxval
				CarryOn = true
			)
			else return b
		)
		else
		(
			SetListenerSel #(-1,-1)
			format "Requires an integer between % and %. \n" minval maxval
			CarryOn = true
		)
	)	
	EscapeEnable = EEnable
)


-- GET YN FUNCTION ******************************************************************************************
-- Function issues supplied prompt to listener. 
-- If user enters y or yes in upper or lower case it returns true
-- if user enters n or no in upper or lower case it returns false
-- if user hits return the function returns the default value which is true if none supplied
-- Parameters:
--	prompt is the value to be sent to the listener window, (string)
--	default is the value the function is to return if the enter key is pressed (true or false)

fn getYN prompt:"Y/N" default:true  = 
(
	local CarryOn = true
	local TR
	local a,b
	local EEnable
	
	EEnable = EscapeEnable
	EscapeEnable = false
	
	do 
	(
		setListenerSel #(-1,-1)
		TR = getKBline prompt:prompt
		a = getListenerSel()
		case of 
		(
			(TR==#Escape): 							(SetListenerSel #(-1,-1) ; return #Escape)
			(TR=="y" or TR=="Y"): 					(CarryOn = false ; Val = true)		-- Y ENTERED
			(TR=="yes" or TR=="YES" or TR=="Yes"):	(CarryOn = false ; Val = true)		-- YES ENTERED
			(TR=="n" or TR=="N"):   				(CarryOn = false ; Val = false)		-- N ENTERED
			(TR=="no" or TR=="NO" or TR=="No"):   	(CarryOn = false ; Val = false)		-- NO ENTERED
			(TR==""):								(CarryOn = false ; Val = default) 	-- RETURN KEY PRESS
			default:
				( 
					b = getListenerSel()
					setListenerSel #((a[1]),b[1])
					SetListenerSelText "Invalid entry... \n" 
					CarryOn = true
				)
		)
	) while CarryOn
	
	EscapeEnable = EEnable
	Val
)

-- GET REAL FUNCTION *****************************************************************************
fn getReal prompt:"Enter value" minval: -1E10 maxval:1E10 default:360.0=
(
	local val,a
	local CarryOn=true
	local EEnable
	
	EEnable = EscapeEnable
	EscapeEnable = false
	
	do
	(
		CarryOn = false
		
		-- GET KEYBOARD INPUT
		val = getKBline prompt:prompt
		
		-- TEST IF ESCAPE KEY PRESSED
		if val == #Escape do (EscapeEnable = true ; return #Escape)
	
		-- TEST IF ENTER KEY PRESSED, IF SO RETURN DEFAULT VALUE
		if (val == "") do (EscapeEnable = EEnable ; return default as float)
			
		-- CONVERT TO FLOAT	
		if (a = val as float) == undefined then 
		(
			setListenerSel #(-1,-1)
			format "\nInvalid input..\n"
			CarryOn = true
		)
		else
		(
			-- TEST IF VALUE FALLS BETWEEN MINIMUM AND MAXIMUM
			if (a >= minval and a <= maxval) then
			(
				EscapeEnable = EEnable
				return a
			)
			else 
			(
				format "Enter a value between % and %...\n" minval maxval
				CarryOn = true
			)
			
		)
	
	) while CarryOn
)		

-- VIEW INQUIRY FUNCTIONS
fn is_ortho_view =
(
	-- get screen space coords of points at different
	-- depths and see if they are perspectively scaled
	local z = mapScreenToView [0,0] 0,
	      z1 = mapScreenToView [0,0] 1000
	z.x == z1.x and z.y == z1.y
)

fn ceq x y = abs (x - y) < 1.0e-5  -- close equality

fn view_type =
(
	if is_ortho_view () then
	(
		local vr = (getViewTM()).rotationPart as EulerAngles
		case of 
		(
			(ceq vr.x 0 and ceq vr.y 0 and ceq vr.z 0): #top
			(ceq vr.x 180 and ceq vr.y 0 and ceq vr.z 0): #bottom
			(ceq vr.x -90 and ceq vr.y 90 and ceq vr.z 0): #left
			(ceq vr.x -90 and ceq vr.y -90 and ceq vr.z 0): #right
			(ceq vr.x -90 and ceq vr.y 0 and ceq vr.z 0): #front
			(ceq vr.x 90 and ceq vr.y 0 and ceq vr.z 180): #back
			default: #user
		)
	)
	else
		#perspective
)

-- *********************************************************	
fn PointConditioner testpt: tolerance: = 
(
	if abs(testpt.x) < tolerance do testpt.x = 0
	if abs(testpt.y) < tolerance do testpt.y = 0
	if abs(testpt.z) < tolerance do testpt.z = 0
	
	testpt
)
	