-- AutoCAD MIRROR COMMAND EMULATION

-- This function mirrors (reflects) shape objects through a plane that:
-- passes through two selected points
-- is parallel to the objects local Z axis

-- Multiple shape objects may be mirrored at the same time however their
-- local Z axes must be parallel to one another. If they are not the script
-- advises the user and exits.

fn _mirror =
(
	-- LOCAL VARIABLES
	local pt1,pt2,pt1_m,pt2_m,delta
	local Item,a,GridObj,SS,WorkSS,MouseSS = #()
	local MirrorCopy,ObjDir
	local dir_vec,dir_vec_w
	local MirrorCS,MIRRORCSW
	local xPart,yPart,zPart,transPart,delta		
	
	-- LOCAL FUNCTION DEFINITIONS
	fn shapeFilt obj = (SuperClassOf obj == shape) --2D mirror..all objects=shapes	
	
	fn MHandler pt2 =
	(
		count = _miror.MouseSS.count
				
		-- DELETE TEMPORARY MIRRORED OBJECTS 
		if count > 0 do 
		(
			delete _miror.MouseSS
			_miror.MouseSS = #()
		)
			
		for obj in _miror.SS do
		(
			NewObj = copy obj
			NewObj.transform = obj.transform
			append _miror.MouseSS NewObj	
		)
			
		-- TRANSFORM PICKPOINTS TO OBJECTS COORDINATE SYSTEM
		_miror.pt1_m = _miror.pt1 * inverse(_miror.a.transform)
		_miror.pt2_m = _miror.pt2 * inverse(_miror.a.transform)
		_miror.pt1_m.z = 0.0
		_miror.pt2_m.z = 0.0
		_miror.delta = _miror.pt2_m - _miror.pt1_m
		
		-- CREATE MATRIX DEFINING MIRROR PLANE BASED ON MIRROR AXIS ENDPOINTS
		_miror.xPart = normalize _miror.delta
		_miror.yPart = [0,0,1]
		_miror.zPart = cross _miror.xPart _miror.yPart
		_miror.transPart = [0,0,0]
		_miror.MirrorCS = matrix3 _miror.xPart _miror.yPart _miror.zPart _miror.transPart
		
		-- CONVERT TO WORLD CS
		_miror.MirrorCSW = _miror.MirrorCS * _miror.a.transform
		_miror.MirrorCSW.row4 = _miror.pt1
		
		-- PERFORM MIRROR (REFLECTION THROUGH GRID PLANE)
		for obj in _miror.MouseSS do 
		(
			in coordsys _miror.MirrorCSW about _miror.MirrorCSW scale obj [1,1,-1]
		)
		redrawViews()		
	)


	-- START CODE BODY *************************************************************
	
	-- DESELECT ANY OBJECT(S) CURRENTLY SELECTED
	EscapeEnable = false
	clearlistener()
	max select none
	
		
	-- LOOP TO ADD ITEMS TO SELECTION SET
	while true do
	(
		Item = pickObject prompt:"Select Objects:" filter:shapeFilt count:1	
		
		if Item==#Escape do 
		(
		setListenerSel #(-1,-1)
		EscapeEnable = true
		return (format "*Cancel*\nOK\n")
		)

		if Item==undefined do exit
		
	 	selectmore Item
		redrawviews()
		format "\n"
	)
	
	format "\n"
	
	if selection.count == 0 do
	(
		setListenerSel #(-1,-1)
		EscapeEnable = true
		return (format "*Cancel*\nOK\n")
	)
	
	-- PULL FIRST OBJECT FROM SELECTION SET 		
	a = selection[1]
	
	-- TEST  IF SELECTED OBJECTS HAVE COMMON PARALLEL VECTORS
	SS = selection as array
	ObjDir = normalize SS[1].dir
	for obj in SS do
	(	
		local TestVal = abs (abs (dot (normalize obj.dir) ObjDir) - 1.0)
		
		if TestVal > 0.001 do 
		(
			messagebox "Cannot proceed, Selected objects must have common local Z direction"
			return redrawviews()
		)
	)
	
	-- PROMPT FOR FIRST MIRROR AXIS END POINT 		
	pt1 = pickpoint prompt:"First point of mirror axis:" snap:#2D
		 
	if (ClassOf pt1 != Point3) do
	(
		max select none
		EscapeEnable=true
		setListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)
	format "\n"
	
	-- PROMPT FOR SECOND MIRROR AXIS END POINT
	pt2 = pickpoint prompt:"Second point:" snap:#2D rubberband:pt1 mouseMoveCallBack:MHandler
		 
	if (ClassOf pt2 != Point3) do
	(
		max select none
		delete MouseSS
		EscapeEnable=true
		setListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)
	format "\n"
				
	redrawviews()
	
	-- TRANSFORM PICKPOINTS TO OBJECTS COORDINATE SYSTEM
	pt1_m = pt1 * inverse(a.transform)
	pt2_m = pt2 * inverse(a.transform)
	pt1_m.z = 0.0
	pt2_m.z = 0.0
	delta = pt2_m - pt1_m
	
	-- CREATE MATRIX DEFINING PLANE BASED ON MIRROR AXIS ENDPOINTS
	xPart = normalize delta
	yPart = [0,0,1]
	zPart = cross xPart yPart
	transPart = [0,0,0]
	MirrorCS = matrix3 xPart yPart zPart transPart
	
	-- CONVERT TO WORLD CS
	MirrorCSW = MirrorCS * a.transform
	MirrorCSW.row4 = pt1
	
	-- PROMPT FOR ORIGINAL OBJECT DELETION
	if (MirrorCopy = getYN prompt:"Delete old objects? <N>" default:false) == #Escape do
	(
		max select none
		SetListenerSel #(-1,-1)
		delete MouseSS
		return (format "*Cancel*\nOK\n")
	)

	-- BUILD NEW SELECTION SET IF ORIGINAL OBJECTS ARE TO BE RETAINED
	if not MirrorCopy then
	(
		local NewObj
		WorkSS = #()
		for obj in SS do
		(
			NewObj = copy obj
			NewObj.transform = obj.transform
			append WorkSS NewObj	
		)
	)
	else	WorkSS = SS
	
	-- PERFORM MIRROR (REFLECTION THROUGH GRID PLANE)
	for obj in WorkSS do 
	(
		in coordsys MirrorCSW about MirrorCSW scale obj [1,1,-1]
	)
	
	delete MouseSS
	max select none
	EscapeEnable = true
	setListenerSel #(-1,-1)
	format "OK\n"
)