-- AUTOCAD ROTATE COMMAND EMULATION
-- Rotates selected objects about a axis that depends on the active viewport
-- and the active grid
-- If the active viewport is an orthographic viewport, (top,front,right,left,bottom,back)
-- the rotation axis will be through the selected base point and in a direction
-- parallel to the view direction. The positive direction of the rotation vector  and towards the user.
-- If the active viewport is non orthographic the rotation axis is through the selected
-- basepoint and parallel to the world Z axis direction.
-- Note that the active viewport is the last viewport the user selects an object in.

-- If objects are selected in multiple viewports the results are based on the final one
-- which may give unexpected results.

fn _rotate =
(
	-- DECLARE LOCALS
	local RotateItems
	local BasePt,RotateAngle
	local RotationVector
	local a,b,UserGrid
	
	-- SYSTEM SETTINGS
	clearListener()
	EscapeEnable = false
	
			
	-- LOOP TO ADD ITEMS TO SELECTION SET
	RotateItems = pickObject prompt:"Select Objects:" count:#multiple select:true
	
	if (ClassOf RotateItems!=array or RotateItems.count==0) do
	(
		EscapeEnable = true
		max select none
		SetListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)
	format "\n"
	
	-- DETERMINE WHETHER USER GRID EXISTS AND IS ACTIVE
	a = getCPTM()	
	b = matrix3 1
	if (a.row1==b.row1 and a.row2==b.row2 and a.row3==b.row3 and a.row4==b.row4) 
	then UserGrid = false
	else UserGrid = true
	
	-- DETERMINE WHAT TRANSFORMATION IS REQD (IF NOT REQD SET TO WCS; MATRIX 3 1)
	if UserGrid 
	then 	RotationVector = a.row3
	else 
	(
		case view_type() of 
		(
			#perspective: 	RotationVector = [0,0,1]
			#user:			RotationVector = [0,0,1]
			#top:			RotationVector = [0,0,1]
			default:		RotationVector = (inverse (getViewTM()) ).row3
		)
	)

	-- PROMPT FOR BASE POINT OF ROTATION		
	BasePt = pickpoint prompt:"Base point:"
		
	if (ClassOf BasePt != Point3) do
	(	EscapeEnable = true
		max select none
		SetListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)
	format "\n"

	-- PROMPT FOR ROTATION ANGLE
	if (RotateAngle = getReal prompt:"Rotation angle: " minval: -360 maxval:360 default:0) == #Escape do
	(
		EscapeEnable = true
		max select none
		setListenerSel #(-1,-1)
		return (format "*Cancel*\nOK\n")
	)
	
	-- ROTATE OBJECTS IN SELECTION ARRAY
	about BasePt in coordsys world rotate RotateItems RotateAngle RotationVector  
	
	
	-- DESELECT SELECTION SET AND REDRAW VIEWPORTS
	max select none
	max views redraw
	EscapeEnable = true
	setListenerSel #(-1,-1)
	format "OK\n"
)
