-- Trim() and Extend() functions

-- OBJECT PICK FILTER FUNCTION
fn SFilter o = superClassOf o == Shape

-- ModName Function ************************************************************************************
fn ModName shape index =
(
	local CountString = index as string
	while CountString.count < 4 do
		CountString = "0" + CountString
	
	(CountString + shape.name)
)

-- CompareShape Function *******************************************************************************	
fn CompareShape Shape1: Shape2: =
(
	local i,j
	local a,b
	
	if (Shape1.numsplines != Shape2.numsplines) do return false
	
	for i = 1 to Shape1.numsplines do
	(
		if (numKnots Shape1 i) != (numKnots Shape2 i) do return false
		
		for j = 1 to (numKnots Shape1 i) do
		(
			a = getKnotPoint Shape1 i j
			b = getKnotPoint Shape2 i j
			if a != b do return false
		)
	)
	return true	
)

-- REMOVE DUPLICATE ARRAY ITEMS FUNCTION ***************************************************************
fn RemoveDuplicates PArray =
(
	local duplicateIndex = #()
	local PArraySize
	local i,j,b,p
	
	PArraySize = PArray.count
	for i = 1 to (PArraySize-1) do
	(
		p = i + 1
		for j = p to PArraySize do
		(
			if PArray[i]==PArray[j] do 
				if findItem duplicateIndex j == 0 do
					append duplicateIndex j
		)
	)
	
	sort duplicateIndex
	b=duplicateIndex.count
	for i = 1 to b do deleteItem PArray duplicateIndex[(b+1-i)]
	return PArray
)

-- TRIM FUNCTION ****************************************************************************************
fn _trim = 
(
	local CuttingEdges = #()
	local Array1=#()
	local Array2=#()
	local Array3=#()
	local Array4=#()
	local a,i,obj,index,NewName,TempObj
	
	-- SYSTEM SETTINGS
	ClearListener()
	EscapeEnable = false
	
	-- SELECT ALL SHAPES
	Array1 = (shapes as array)	
	
	-- FILTER VISIBLE SHAPES FROM ARRAY1
	for obj in Array1 where (obj.ishidden==false) do
	(
		append Array2 obj
		obj.ishidden = true
	)
		
	-- CONVERT ARRAY2 SHAPES TO SPLINESHAPES AND APPEND TO ARRAY3
	i = 1
	for obj in Array2 do
	(
		a=copy obj
		a.name = ModName obj i
		i+=1
		a.wirecolor = obj.wirecolor
		converttosplineshape a
		updateshape a
		append Array3 a
	)
	redrawviews()
		
	-- PROMPT FOR SELECTION OF THE CUTTING EDGE(S)
	CuttingEdges = pickObject prompt:"Select cutting edges:" count:#multiple filter:SFilter select:true

	-- CHECK FOR ESCAPE KEY PRESS OR EMPTY SELECTION SET
	if (ClassOf CuttingEdges != array or CuttingEdges.count == 0) do
	(
		max select none
		delete Array3									-- deletes splineshape copies of original shapes
		for obj in Array2 do obj.ishidden=false			-- set original shapes to visible
		redrawviews()
		setListenerSel #(-1,-1)
		EscapeEnable = true
		return (format "\n*Cancel*\nOK\n")
	)
	format "\n"
	
	CuttingEdges = RemoveDuplicates CuttingEdges

	-- PROMPT FOR OBJECTS TO TRIM AND DESELECT OBJECTS
	format "Select object to trim:\n"
	max select none
		
	-- PERFORM TRIM OPERATION ON SPLINESHAPE COPIES OF ORIGINAL SHAPES
try (
	if (trimExtend CuttingEdges Array3 project:#grid infinite:false trim:true extend:false) == #Escape do
	(
		delete Array3									-- deletes splineshape copies of original shapes
		for obj in Array2 do obj.ishidden=false			-- set original shapes to visible
		redrawviews()
		setListenerSel #(-1,-1)
		EscapeEnable = true
		return (format "*Cancel*\nOK\n")
	)
) catch()

	setWaitCursor()
	
	-- LOOP THROUGH ARRAYS TO COMPARE SHAPES
	for i = 1 to (Array2.count) do
	(
		-- IF ORIGINAL SHAPE NO LONGER IN ARRAY3...DELETE IT,(THIS SHOULD NOT NORMALLY OCCUR)
		if (index=(findItem Array3 (execute ("$" + (newName=modName Array2[i] i))))) == 0 then
		(
			append Array4 Array2[i]
		)
		else
		-- COMPARE THE ORIGINAL SHAPE TO THE NEW ONE TO SEE IF TRIM OPERATION CHANGED IT
		(
			tempObj = converttosplineshape (copy Array2[i])
			
			-- COLLAPSE STACK ON CURRENT SHAPE (REQUIRED TO OBTAIN CORRECT GETKNOTPOINT VALUES)
			collapseStack Array3[index]
			
			-- DETERMINE IF ENTIRE SHAPE WAS TRIMMED AWAY
			if Array3[index].numsplines == 0 then
			(
				append Array4 Array3[index]
				append Array4 Array2[i]
				delete TempObj
			)
			else
			(
				if (CompareShape Shape1:tempObj Shape2:Array3[index]) then
				-- THEY ARE THE SAME...MARK COPY FOR DELETION AND MAKE ORIGINAL VISIBLE
				(
					append Array4 Array3[index]
					Array2[i].ishidden = false
					delete TempObj
				)
				-- THEY ARE DIFFERENT...MARK ORIGINAL FOR DELETION
				else
				(
					Array3[index].name = Array2[i].name
					append Array4 Array2[i]
					delete tempObj
				)
			)
		)
	)
	
	setArrowCursor()
	
	-- DELETE SHAPES MARKED FOR DELETION
	delete Array4
	
	max select none
	redrawviews()
	EscapeEnable = true
	SetListenerSel #(-1,-1)
	format "OK\n"
)

-- Extend function ****************************************************************************************
fn _extend  = 
(
	local BoundaryEdges = #()
	local ExtendItems = #()
			
	-- SYSTEM SETTINGS
	ClearListener()
	EscapeEnable = false
	
	-- SELECT THE BOUNDARY EDGE(S)
	BoundaryEdges = pickObject prompt:"Select boundary edges:" count:#multiple filter:SFilter select:true
	
	-- CHECK FOR ESCAPE KEY PRESS OR EMPTY SELECTION SET
	if (ClassOf BoundaryEdges != Array or BoundaryEdges.count == 0) do
	(
		max select none
		setListenerSel #(-1,-1)
		EscapeEnable = true
		return (format "*Cancel*\nOK\n")
	)
	format "\n"
		
	-- SELECT ALL SHAPES AS ITEMS ELIGIBLE FOR EXTEND
	ExtendItems = (shapes as array)	
	
	-- DESELECT OBJECTS
	max select none
	
	-- PROMPT FOR OBJECTS TO EXTEND
	format "Select object to extend:\n"

	-- EXTEND FUNCTION
try (
	if (trim_Extend BoundaryEdges ExtendItems project:#grid infinite:false trim:false extend:true)==#Escape do
	(
		max select none
		setListenerSel #(-1,-1)
		EscapeEnable = true
		return (format "*Cancel*\nOK\n")
	)
)catch(print "extend:trim_extend error")

	EscapeEnable = true
	setListenerSel #(-1,-1)
	format "OK\n"
)
